"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Leaf = void 0;
const peer_1 = require("./peer");
const node_1 = require("./node");
const util_1 = require("../util");
const err_1 = require("../err");
class Leaf {
    constructor(peer, pk, sk) {
        this.peer = peer;
        this.pk = pk;
        this.sk = sk;
    }
    clone() {
        var _a;
        return new Leaf(this.peer.clone(), this.pk.slice(), (_a = this.sk) === null || _a === void 0 ? void 0 : _a.slice());
    }
    static me(me, init) {
        if (init) {
            const { pk, sk } = (0, util_1.keypair)((0, util_1.keyExchangeReceive)(me.identity, init.identity_pk, me.ephemeral, init.exchange_pk));
            return new Leaf(me, pk, sk);
        }
        else {
            return new Leaf(me, me.ephemeral.pk, me.ephemeral.sk);
        }
    }
    static member(member, init) {
        if (init instanceof Uint8Array) {
            return new Leaf(member, init);
        }
        else {
            const { pk, sk } = (0, util_1.keypair)((0, util_1.keyExchangeSend)(init.identity, member.identity_pk, init.exchange, member.ephemeral_pk));
            return new Leaf(member, pk, sk);
        }
    }
    get length() {
        return 1;
    }
    get isOnMyPath() {
        return this.peer.isMy;
    }
    split(at, peer, init) {
        if ((0, util_1.equals)(at, this.pk)) {
            if (Array.isArray(init)) {
                const leaf_pk = init.shift();
                const node_pk = init.shift();
                if (leaf_pk === undefined || node_pk === undefined) {
                    throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, 'Not enough PKs to process split');
                }
                return new node_1.Node(this, Leaf.member(peer, leaf_pk), node_pk);
            }
            else if (peer instanceof peer_1.Me && 'identity_pk' in init) {
                return new node_1.Node(this, Leaf.me(peer, init));
            }
            else if ('identity' in init) {
                return new node_1.Node(this, Leaf.member(peer, init));
            }
            throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, "Can't create a member leaf with this init");
        }
        return undefined;
    }
    update(pk) {
        this.pk = pk;
    }
    *members() {
        yield this;
    }
    *leafs() {
        yield this;
    }
}
exports.Leaf = Leaf;
