"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Node = void 0;
const __1 = require("..");
const err_1 = require("../err");
const util_1 = require("../util");
const leaf_1 = require("./leaf");
class Node {
    constructor(left, right, pk) {
        this.left = left;
        this.right = right;
        if (this.left.sk || this.right.sk) {
            this.pk = this.derive();
        }
        else if (pk) {
            this.pk = pk;
        }
        else {
            throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, "Can't construct node not on my path without PK");
        }
    }
    clone() {
        var _a;
        return new Node(this.left.clone(), this.right.clone(), (_a = this.pk) === null || _a === void 0 ? void 0 : _a.slice());
    }
    update(pk) {
        if (this.isOnMyPath) {
            this.derive();
            if (!(0, util_1.equals)(this.pk, pk)) {
                throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, 'Updated PK is not equal to derived one');
            }
        }
        else {
            this.pk = pk;
        }
    }
    derive() {
        const sk = this.left.sk || this.right.sk;
        if (sk) {
            const pk = this.left.sk ? this.right.pk : this.left.pk;
            const keys = Node.derive(sk, pk);
            this.sk = keys.sk;
            this.pk = keys.pk;
            return this.pk;
        }
        else {
            throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, "Can't construct node not on my path without PK");
        }
    }
    static derive(sec, pub) {
        const sk = __1.crypt.hkdf(__1.crypt.getSharedSecret(sec, pub), undefined, undefined, __1.crypt.SK_LENGTH);
        const pk = __1.crypt.derivePublicKey(sk);
        return { sk, pk };
    }
    get length() {
        return this.left.length + this.right.length + 1;
    }
    get isOnMyPath() {
        return this.left.isOnMyPath || this.right.isOnMyPath;
    }
    split(at, peer, init) {
        const left = this.left.split(at, peer, init);
        if (left instanceof Node) {
            this.left = left;
        }
        const right = this.right.split(at, peer, init);
        if (right instanceof Node) {
            this.right = right;
        }
        if (left || right) {
            if (Array.isArray(init)) {
                const t = init.shift();
                if (t === undefined) {
                    throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, 'Not enough PKs to process split');
                }
                this.pk = t;
            }
            if (this.left.sk || this.right.sk) {
                this.derive();
            }
        }
        return !!left || !!right;
    }
    *members() {
        yield this;
        for (const n of this.left.members()) {
            yield n;
        }
        for (const n of this.right.members()) {
            yield n;
        }
    }
    *leafs() {
        for (const l of this.left.leafs()) {
            yield l;
        }
        for (const l of this.right.leafs()) {
            yield l;
        }
    }
    *pathOf(peer) {
        let yelded = false;
        if (this.left instanceof leaf_1.Leaf) {
            if (this.left.peer === peer) {
                yield this.left;
                yelded = true;
            }
        }
        else {
            for (const l of this.left.pathOf(peer)) {
                yelded = true;
                yield l;
            }
        }
        if (this.right instanceof leaf_1.Leaf) {
            if (this.right.peer === peer) {
                yield this.right;
                yelded = true;
            }
        }
        else {
            for (const l of this.right.pathOf(peer)) {
                yelded = true;
                yield l;
            }
        }
        if (yelded) {
            yield this;
        }
    }
}
exports.Node = Node;
