"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializeTree = exports.serializeTree = exports.cleanupTree = exports.createTree = void 0;
const __1 = require("..");
const err_1 = require("../err");
const leaf_1 = require("./leaf");
const node_1 = require("./node");
const peer_1 = require("./peer");
function createTree(peers, init) {
    const initiator = peers[0];
    if (!initiator) {
        throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, 'Null first per');
    }
    const tree = createTreeRecurcive(peers, init);
    cleanupTree(tree, initiator);
    return tree;
}
exports.createTree = createTree;
function cleanupTree(tree, peer) {
    const my = Array.from(tree.pathOf(peer));
    for (const member of tree.members()) {
        if (!my.includes(member)) {
            member.sk = undefined;
        }
    }
}
exports.cleanupTree = cleanupTree;
function createTreeRecurcive(peers, init) {
    switch (peers.length) {
        case 0:
            throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, 'No peers');
        case 1:
            if (peers[0] instanceof peer_1.Me) {
                return leaf_1.Leaf.me(peers[0]);
            }
            else if (peers[0] instanceof peer_1.Peer) {
                return leaf_1.Leaf.member(peers[0], init);
            }
            else {
                throw new err_1.Err(err_1.ErrCode.INCONSISTENT_STATE, 'Unreachable');
            }
        default: {
            const l = leftTreeSize(peers.length);
            const left = createTreeRecurcive(peers.slice(0, l), init);
            const right = createTreeRecurcive(peers.slice(l, peers.length), init);
            return new node_1.Node(left, right);
        }
    }
}
function leftTreeSize(numLeaves) {
    return Math.pow(2, Math.ceil(Math.log(numLeaves) / Math.log(2)) - 1);
}
function serializeTree(data, view, offset, node) {
    if (node instanceof leaf_1.Leaf) {
        view.setUint8(offset.add(1), 0);
        data.set(node.pk, offset.add(__1.crypt.PK_LENGTH));
        data.set(node.peer.identity_pk, offset.add(__1.crypt.PK_LENGTH));
        data.set(node.peer.ephemeral_pk, offset.add(__1.crypt.PK_LENGTH));
        data.set(node.peer.ephemeral_signature, offset.add(__1.crypt.SI_LENGTH));
    }
    else {
        view.setUint8(offset.add(1), 1);
        data.set(node.pk, offset.add(__1.crypt.PK_LENGTH));
        serializeTree(data, view, offset, node.left);
        serializeTree(data, view, offset, node.right);
    }
}
exports.serializeTree = serializeTree;
function deserializeTree(data, view, offset) {
    const kind = view.getUint8(offset.add(1));
    const pk = data.slice(offset.add(__1.crypt.PK_LENGTH), offset.value);
    if (kind === 0) {
        const identity_pk = data.slice(offset.add(__1.crypt.PK_LENGTH), offset.value);
        const ephemeral_pk = data.slice(offset.add(__1.crypt.PK_LENGTH), offset.value);
        const ephemeral_signature = data.slice(offset.add(__1.crypt.SI_LENGTH), offset.value);
        return leaf_1.Leaf.member(new peer_1.Peer(identity_pk, ephemeral_pk, ephemeral_signature), pk);
    }
    else {
        const left = deserializeTree(data, view, offset);
        const right = deserializeTree(data, view, offset);
        return new node_1.Node(left, right, pk);
    }
}
exports.deserializeTree = deserializeTree;
