import { Leaf, Node } from '.';
import { Keypair, PK, SK } from './types';
export declare function keypair(sk?: SK): Keypair;
/**
 * X3DH send
 */
export declare function keyExchangeSend(selfIdentity: Keypair, remoteIdentity: PK, keyExchangeKeyPair: Keypair, remoteEphemeralKey: PK): Uint8Array;
/**
 * X3DH receive
 */
export declare function keyExchangeReceive(selfIdentity: Keypair, remoteIdentity: PK, ephemeralKey: Keypair, keyExchangeKey: PK): Uint8Array;
/**
 * Concat several arrays into one
 */
export declare function concat(...arrays: Uint8Array[]): Uint8Array;
/**
 * Split big array into several smaller ones of length item_length.
 * Does alloc for each subarray.
 */
export declare function split(data: Uint8Array, offset: Offset, item_length: number, item_count?: number): Uint8Array[];
/**
 * Compare two arrays
 */
export declare function equals(a: Uint8Array, b: Uint8Array): boolean;
/**
 * Stringify Uint8Array into hex form
 *
 * @param array array to convert
 * @returns hex string of array
 */
export declare function toHex(array: Uint8Array): string;
/**
 * Print three first numbers of Uint8Array
 *
 * @param array array to convert
 * @returns hex string of array
 */
export declare function toFirstThree(array: Uint8Array): string;
/**
 * Just a handy class for arrays parsing, returns old value when adding to it
 */
export declare class Offset {
    private counter;
    constructor(counter?: number);
    get value(): number;
    add(n: number): number;
    assert(n: number): void;
}
export declare const Gen: {
    collect<G, T>(g: Generator<G, any, unknown>, f: (g: G) => T): T[];
    find<G_1>(g: Generator<G_1, any, unknown>, f: (g: G_1) => boolean): G_1 | undefined;
};
export declare function printTree(node: Node | Leaf, level?: number, toString?: typeof toFirstThree): void;
