"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.messageSignatures = void 0;
const src_1 = require("../../src");
const test_utils_1 = require("../test.utils");
function messageSignatures() {
    it('should not accept setup message with invalid data or signature', () => {
        const [a, b] = (0, test_utils_1.referenceGroup)();
        const invalidSetupMessage = a.setupMessage.slice();
        invalidSetupMessage[100] = (invalidSetupMessage[100] || 0) + 1;
        expect(() => src_1.ART.fromSetupMessage(b.me, invalidSetupMessage)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
    });
    it('should not accept update key message with invalid data or signature', () => {
        const [a, b] = (0, test_utils_1.referenceGroup)();
        const update = b.updateKey();
        const invalidUpdate = update.slice();
        invalidUpdate[10] = (invalidUpdate[10] || 0) + 1;
        expect(() => a.processKeyUpdate(invalidUpdate)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
        const malicious = (0, src_1.keypair)();
        const substitutedUpdate = update.slice();
        substitutedUpdate.set(src_1.crypt.sign(substitutedUpdate.slice(0, -src_1.crypt.SI_LENGTH), malicious.sk), substitutedUpdate.length - src_1.crypt.SI_LENGTH);
        expect(() => a.processKeyUpdate(substitutedUpdate)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
    });
    it('should not accept update key message with invalid data or signature', () => {
        var _a;
        const [a, , c] = (0, test_utils_1.referenceGroup)();
        const d = (0, test_utils_1.generate)(1)[0];
        if (!d) {
            throw new Error();
        }
        const { message, snapshot } = a.split(new src_1.Peer(d.identity.pk, d.ephemeral.pk, d.ephemeral_signature), ((_a = a.leafOf(c.me.identity_pk)) === null || _a === void 0 ? void 0 : _a.pk) || new Uint8Array());
        const invalidMessage = message.slice();
        const invalidSnapshot = snapshot.slice();
        invalidMessage[100] = (invalidMessage[100] || 0) + 1;
        invalidSnapshot[100] = (invalidSnapshot[100] || 0) + 1;
        expect(() => a.processSplit(invalidMessage)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
        expect(() => src_1.ART.fromSplitMessage(new src_1.Me(d.identity, d.ephemeral), invalidSnapshot, message)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
        expect(() => src_1.ART.fromSplitMessage(new src_1.Me(d.identity, d.ephemeral), snapshot, invalidMessage)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
        const malicious = (0, src_1.keypair)();
        const substitutedSnapshot = snapshot.slice();
        const substitutedMessage = message.slice();
        substitutedMessage.set(src_1.crypt.sign(substitutedMessage.slice(0, -src_1.crypt.SI_LENGTH), malicious.sk), substitutedMessage.length - src_1.crypt.SI_LENGTH);
        substitutedSnapshot.set(src_1.crypt.sign(substitutedSnapshot.slice(0, -src_1.crypt.SI_LENGTH), malicious.sk), substitutedSnapshot.length - src_1.crypt.SI_LENGTH);
        expect(() => a.processSplit(substitutedMessage)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
        expect(() => src_1.ART.fromSplitMessage(new src_1.Me(d.identity, d.ephemeral), substitutedSnapshot, message)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
        expect(() => src_1.ART.fromSplitMessage(new src_1.Me(d.identity, d.ephemeral), snapshot, substitutedMessage)).toThrow(expect.objectContaining({ code: src_1.ErrCode.INVALID_SIGNATURE }));
    });
}
exports.messageSignatures = messageSignatures;
