"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aes_1 = require("@noble/ciphers/aes");
const secp256k1_1 = require("@noble/curves/secp256k1");
const hkdf_1 = require("@noble/hashes/hkdf");
const sha256_1 = require("@noble/hashes/sha256");
const crypto_1 = require("crypto");
const src_1 = require("../src");
beforeEach(function () {
    if (expect.getState().testPath.includes('/no-crypt.spec.ts')) {
        return;
    }
    (0, src_1.setCrypto)({
        generateSecretKey: secp256k1_1.secp256k1.utils.randomPrivateKey,
        derivePublicKey: secp256k1_1.secp256k1.getPublicKey,
        getSharedSecret: secp256k1_1.secp256k1.getSharedSecret,
        hash_256: sha256_1.sha256,
        sign: function (data, sk) {
            return secp256k1_1.secp256k1.sign(data, sk, { prehash: true }).toCompactRawBytes();
        },
        verify: function (si, data, pk) {
            return secp256k1_1.secp256k1.verify(si, data, pk, { prehash: true });
        },
        hkdf: hkdf_1.hkdf.bind(null, sha256_1.sha256),
        encrypt: function (data, key) {
            const nonce = (0, crypto_1.randomBytes)(this.ENCRYPTION_PREFIX_LENGTH);
            return (0, src_1.concat)(nonce, (0, aes_1.gcm)(key, nonce).encrypt(data));
        },
        decrypt: function (data, key) {
            const nonce = data.subarray(0, this.ENCRYPTION_PREFIX_LENGTH);
            try {
                return (0, aes_1.gcm)(key, nonce).decrypt(data.subarray(this.ENCRYPTION_PREFIX_LENGTH));
            }
            catch (_a) {
                return null;
            }
        },
        PK_LENGTH: 33,
        SK_LENGTH: 32,
        SI_LENGTH: 64,
        ENCRYPTION_PREFIX_LENGTH: 12,
        ENCRYPTION_SUFFIX_LENGTH: 16,
    });
});
